/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SMTP_Client.c
 * Purpose: SMTP Client example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */

#include "SCU_LPC43xx.h"                // Keil::Device:SCU
#include "GPIO_LPC43xx.h"               // Keil::Device:GPIO

#include "Board_LED.h"

// Enter your valid SMTP Server IP address here
const uint8_t srv_ip[4] = { 192,168,0,253 };

uint8_t smtp_event;
osThreadId send_id;

static void BlinkLed (void const *arg);
static void SendMail (void const *arg);

osThreadDef(BlinkLed, osPriorityNormal, 1, 0);
osThreadDef(SendMail, osPriorityNormal, 1, 0);

/// Read analog inputs
uint16_t AD_in (uint32_t ch) {
  static int32_t val = 0;

  val++;
  val &= 0x3FFU;

  return (val);
}

/// Inform application about SMTP status
void smtp_inform (smtpClientEvent event) {
  smtp_event = event;
  // Notify send thread of SMTP status
  osSignalSet (send_id, 0x0001);
}

/*----------------------------------------------------------------------------
  Thread 'SendMail': Send an email
 *---------------------------------------------------------------------------*/
static void SendMail (void const *arg) {
  netStatus retv;

  send_id = osThreadGetId ();

  while(1) {
    // After 5 seconds start sending email
    osDelay (5000);

    // Start SMTP client, use default SMTP port
    retv = smtp_client_connect (srv_ip, 0);
    if (retv != netOK) {
      // Failed to start SMTP process
      continue;
    }
    osSignalWait (0x0001, osWaitForever);
    // Check the completion code
    switch (smtp_event) {
      case smtpClientSuccess:
        // Success, terminate thread
        osThreadTerminate (send_id);
        break;
      case smtpClientTimeout:
        // Timeout, try again
      break;
      case smtpClientError:
        // Error, try again
        break;
    }
  }
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[4] = { 0x01,0x03,0x02,0x00,};
  int cnt = 0;

  while(1) {
    // Every 100 ms
    LED_SetOut (led_val[cnt]);
    if (++cnt >= sizeof(led_val)) {
      cnt = 0;
    }
    osDelay (100);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {

  /* LAN8720 on LPC430-Xplorer uses a GPIO for nRST */
  GPIO_PortClock (1);
  GPIO_SetDir   (0, 4, GPIO_DIR_OUTPUT);
  GPIO_PinWrite (0, 4, 1);
  SCU_PinConfigure (1, 0, SCU_CFG_MODE_FUNC0);

  LED_Initialize ();
  net_initialize ();

  osThreadCreate (osThread(BlinkLed), NULL);
  osThreadCreate (osThread(SendMail), NULL);

  while(1) {
    net_main ();
    osThreadYield ();
  }
}
